/*
 * Ensys srl
 * 
 * Questo esempio recupera i dati raw dall'accelerometro installato nella IndRsp
 * 
 * Verificare di aver abilitato la porta I2C0, ad esempio integrando nel file
 * 	/boot/config.txt 
 * le seguenti righe:
 * 	#i2c0 - periferiche
 * 	dtparam=i2c0=on
 * 	dtoverlay=i2c0-bcm2708,pins_28_29
 * 
 * Per compilare questo file utilizzare il comando:
 * 	gcc -o test lis3de.c
 * Poi eseguire:
 * 	./test
 */
 

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <linux/i2c-dev.h>

#define HD__DEV__ADDR       (0x29u)
#define HD__ID__ADDR        (0x0Fu)
#define HD__ID__VAL         (0x33u)

int32_t i2c__byte_read_byte(int32_t fd, uint8_t add, uint8_t *pdata){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,pdata,1u)==1u){
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_read_buffer(int32_t fd, uint8_t add, uint8_t buffer[], uint8_t dim_buffer){
        int32_t ritorno = -1;
        if(write(fd,&add,1u)==1u){
                if(read(fd,buffer,dim_buffer)==dim_buffer){
                        ritorno = 0;
                }
        }
        return ritorno;
}
int32_t i2c__byte_write_byte(int32_t fd, uint8_t add, uint8_t data){
        uint8_t buffer[2];
        int32_t ritorno = -1;
        buffer[0]=add;
        buffer[1]=data;
        if(write(fd,buffer,2u)==2u){
                ritorno = 0;
        }
        return ritorno;
}

int32_t main(int32_t argc, uint8_t *argv[])
{
	int32_t fd;
        uint8_t data;
        uint8_t buffer[3];
        
        /* Apertura device */
       	if((fd=open("/dev/i2c-0", O_RDWR))<0)           printf("Error open\n");
        if(ioctl(fd, I2C_SLAVE, HD__DEV__ADDR)<0)        printf("Error set addr\n");
        
        /* Lettura ID */
        if(i2c__byte_read_byte(fd, HD__ID__ADDR, &data)==0){
                if(data==HD__ID__VAL)      printf("OK, trovato LIS3DE!\n");
        }
        
        /* Set data rate */
        if(i2c__byte_write_byte(fd, 0x20, 0x27)==0){
                printf(" 0x%02x\n", data);
        }

        for(int32_t i=0;i<100;){
                if(i2c__byte_read_byte(fd, 0x27u, &data)!=0){
                        printf("Errore\n");
                        break;
                }
                else{
                        if(data){
                                if(i2c__byte_read_byte(fd, 0x29, &buffer[0])==0){
                                        printf(" %03d ", buffer[0]);
                                }
                                if(i2c__byte_read_byte(fd, 0x2B, &buffer[1])==0){
                                        printf(" %03d ", buffer[1]);
                                }
                                if(i2c__byte_read_byte(fd, 0x2D, &buffer[2])==0){
                                        printf(" %03d ", buffer[2]);
                                }
                                
                                printf(" %02x \n", data);
                                
                                i++;
                        }
                }               
        }
        
        /* Chiusura device */
        close(fd);

	return (0);
}
